/*
 * File:   CAi.cpp
 * Author: Adam Netocny
 *
 * Created on Pondelok, 2009, december 28, 17:50
 */

#include "CAi.h"
#include "COponent.h"
#include "CTable.h"
#include <math.h>
#include <stdlib.h>
#include <iostream>

using namespace std;
//Konstruktor, alokuje nove objekty pre vypocet hodnot

CAi::CAi(CTable* table) : COponent(table) {
    m_Obrana = new CTable(table->GetSize());
    m_Utok = new CTable(table->GetSize());
}

//Destruktor, zrusi tabulky pre vypocet

CAi::~CAi(void) {
    delete m_Obrana;
    delete m_Utok;
}

//Vynuluje hodnoty v tabulkach pre vypocet

void CAi::Vynuluj(void) {
    for (int i = 0; i < m_Table->GetSize(); i++) {
        for (int j = 0; j < m_Table->GetSize(); j++) {
            m_Obrana->ClearChar(i, j);
            m_Utok->ClearChar(i, j);
        }
    }
}

//Konstanta rozsahu vyratanych hodnot
#define MAXPOMER 100

//Vypocita hodnoty pre utocnu tabulku

void CAi::VyhodnotUtok(void) {
    //Maximalna hodnota
    int max = 8 * ((m_Table->GetSize() < 5) ? m_Table->GetSize() - 1 : 4)
            *((m_Table->GetSize() < 5) ? m_Table->GetSize() - 1 : 4)
            *((m_Table->GetSize() < 5) ? m_Table->GetSize() - 1 : 4);

    //prejde kazdy prvok tabulky
    for (int i = 0; i < m_Table->GetSize(); i++) {
        for (int j = 0; j < m_Table->GetSize(); j++) {
            //urci ci nie je obsadene
            char tmp;
            m_Table->GetChar(i, j, tmp);
            if (tmp == '\0') {
                //vypocita pocet suvyslich blokov vsetkymi 8 smermy s preferenciou dlzsich
                int utok = 0;
                //urci sme
                for (int a = -1; a < 2; a++) {
                    for (int b = -1; b < 2; b++) {
                        int Pocet = 1;
                        //hlada suvisly vlok
                        while (true) {
                            if (!m_Table->GetChar(i + Pocet * a, j + Pocet * b, tmp)) break;
                            if (tmp != 'O') break;
                            Pocet++;
                        }
                        //pripocita s preferenciou dlhsich, p^3 jasne odlisi dlhsie
                        utok += Pocet * Pocet * Pocet;
                    }
                }
                //zapise vypocitanu hodnotu do tabulky
                m_Utok->SetChar(i, j, (char) (((double) MAXPOMER * utok) / ((double) max)));
            }
        }
    }
}

//Vypocita hodnoty pre obrannu tabulku

void CAi::VyhodnotObranu(void) {
    //Maximalna hodnota
    int max = 8 * ((m_Table->GetSize() < 5) ? m_Table->GetSize() - 1 : 4)
            *((m_Table->GetSize() < 5) ? m_Table->GetSize() - 1 : 4)
            *((m_Table->GetSize() < 5) ? m_Table->GetSize() - 1 : 4);

    //prejde kazdy prvok tabulky
    for (int i = 0; i < m_Table->GetSize(); i++) {
        for (int j = 0; j < m_Table->GetSize(); j++) {
            //urci ci nie je obsadene
            char tmp;
            m_Table->GetChar(i, j, tmp);
            if (tmp == '\0') {
                //vypocita pocet suvyslich blokov vsetkymi 8 smermy s preferenciou dlzsich
                int obrana = 0;
                //urci sme
                for (int a = -1; a < 2; a++) {
                    for (int b = -1; b < 2; b++) {
                        int Pocet = 1;
                        //hlada suvisly vlok
                        while (true) {
                            if (!m_Table->GetChar(i + Pocet * a, j + Pocet * b, tmp)) break;
                            if (tmp != 'X') break;
                            Pocet++;
                        }
                        //pripocita s preferenciou dlhsich, p^3 jasne odlisi dlhsie
                        obrana += Pocet * Pocet * Pocet;
                    }
                }
                //zapise vypocitanu hodnotu do tabulky
                m_Obrana->SetChar(i, j, (char) (((double) MAXPOMER * obrana) / ((double) max)));
            }
        }
    }
}

//prehodnoti podla randomizovaneho pomeru, najvhodnejsi tah

void CAi::VratTah(int& x, int& y) {
    double max = 0;
    char a, b;
    double pomer;

    //randomizuje pomer
    double rndu = ((double) rand() / RAND_MAX)*((double) rand() / RAND_MAX)*0.8;
    double rndo = ((double) rand() / RAND_MAX)*((double) rand() / RAND_MAX)*1.2;

    //prejde vsetky polia oboch tabuliek
    for (int i = 0; i < m_Table->GetSize(); i++) {
        for (int j = 0; j < m_Table->GetSize(); j++) {
            //ziska hodnoty v tabulkach
            m_Obrana->GetChar(i, j, a);
            m_Utok->GetChar(i, j, b);

            //podla randomizovaneho pomeru urci vyslednu hodnotu policka
            pomer = rndo * ((double) a) + rndu * ((double) b);

            //ak je hodnota vacsia ako maximum nahradi maximum
            if (max < pomer) {
                x = i;
                y = j;
                max = pomer;
            }
        }
    }
}

//Vypocita a zapise jeden tah

bool CAi::MakeMove(void) {
    //kontrola vyhry/remizy
    if (m_Table->Check() != '\0') return true;

    //Vynuluje a vyrata nanovo hodnoty
    Vynuluj();
    VyhodnotObranu();
    VyhodnotUtok();

    //Ziska poziciu noveho tahu
    int x, y;
    VratTah(x, y);

    //Zapise tah
    m_Table->SetChar(x, y, 'O');
    return true;
}

//Urci ci je mozne vratit hru o krok spat

bool CAi::AllowsBack() {
    return true;
}