/*
 * File:   CNetwork.cpp
 * Author: Adam Netocny
 *
 * Created on Pondelok, 2009, december 28, 17:53
 */

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <iostream>
#include <stdlib.h>
#include <strings.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "CNetwork.h"
#include "COponent.h"
#include "CGame.h"

//Konstantna velkost bufferu 1 byte suradnica x, 1 byte suradnica y(cislo 1-9)
#define	BUFFSIZE 2

//Konstruktor, prichysta objekt na pripojenie

CNetwork::CNetwork(CTable* table, char* host, int port, bool server, CGame* game) : COponent(table) {
    m_Host = host;
    m_Port = port;
    m_Server = server;
    m_game = game;
    c_sockfd = -1;
}

//Zrusi zdroje a uzavrie spojenie

CNetwork::~CNetwork(void) {
    if (c_sockfd != -1) {
        char buf[BUFFSIZE];
        int mlen = 2;

        buf[0] = 'E';
        buf[1] = 'X';

        send(c_sockfd, buf, mlen, 0);
    }
    delete[] m_Host;
    close(c_sockfd);
}

//Metoda zabezpecuje spojenie klient server

bool CNetwork::Connect(int & gridSize) {
    //Ak ide o server
    if (m_Server) {
        //pripravim struktury
        int sockfd;
        struct sockaddr_in my_addr, rem_addr;
        socklen_t rem_addr_length;

        //vytvori soket
        if ((sockfd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
            return false;
        }

        //pripravi adresnu strukturu
        bzero(&my_addr, sizeof (my_addr));
        my_addr.sin_family = AF_INET;
        my_addr.sin_port = htons(m_Port);

        /**
         * Ziska a vypise IPcku
         */
        try {
            char szHostName[255];
            gethostname(szHostName, 255);
            std::cout << "Server bezi na pocitaci: " << szHostName << std::endl;
        } catch (...) {
        }

        //Pripoji soket
        if (bind(sockfd, (struct sockaddr *) & my_addr, sizeof (my_addr)) == -1) {
            close(sockfd);
            return false;
        }

        //zmeni stav soketu na nacuvajuci
        if (listen(sockfd, 5) == -1) {
            close(sockfd);
            return false;
        }

        //caka na pripojenie klienta
        rem_addr_length = sizeof (rem_addr);
        if ((c_sockfd = accept(sockfd, (struct sockaddr *) & rem_addr, &rem_addr_length)) == -1) {
            close(c_sockfd);
            return false;
        }

        //prijal spojenie od klienta
        //odosle velkost pola
        char buf1[1];
        int mlen = 1;
        buf1[0] = (char) gridSize;
        if (send(c_sockfd, buf1, mlen, 0) == -1) {
            close(c_sockfd);
            return false;
        }
    } else {
        //Ak klient
        //pripravi struktury
        struct sockaddr_in sin;

        //vytvori soket
        if ((c_sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
            return false;
        }

        //pripravi adresnu strukturu
        bzero(&sin, sizeof (sin));
        sin.sin_family = AF_INET;
        sin.sin_port = htons(m_Port);
        //ziska adresu z hostname(alebo ip), ochrana pre dvojake zadanie
        try {
            char * szLocalIP;
            if (gethostbyname(m_Host) != NULL) {
                szLocalIP = inet_ntoa(*(struct in_addr *) * gethostbyname(m_Host)->h_addr_list);
                sin.sin_addr.s_addr = inet_addr(szLocalIP);
            } else {
                return false;
            }
        } catch (...) {
            return false;
        }

        //pokusi sa pripojit na server
        if (connect(c_sockfd, (struct sockaddr *) & sin, sizeof (sin)) == -1) {
            close(c_sockfd);
            return false;
        }

        /**
         *
         * Prijmem velkost hracieho pola :)
         *
         */
        char buf1[1];
        int mlen = 0;
        while (mlen != 1) {
            if ((mlen = recv(c_sockfd, buf1, BUFFSIZE, 0)) == -1) {
                close(c_sockfd);
                return false;
            }
        }

        //velkost prijata
        gridSize = (int) buf1[0];
        std::cout << "Velkost pola: " << gridSize << std::endl;
        m_Table->SetSize(gridSize);

        /**
         *
         * Prijmem prvy tah :)
         *
         */
        std::cout << "Prijmam prvy tah..." << std::endl;
        char buf[BUFFSIZE];
        mlen = 0;
        while (mlen != BUFFSIZE) {
            if ((mlen = recv(c_sockfd, buf, BUFFSIZE, 0)) == -1) {
                close(c_sockfd);
                return false;
            }
        }

        if (buf[0] == 'E' && buf[1] == 'X') {
            return false;
        }

        //spracuje prvy tah
        m_Table->SetChar((int) (buf[0] - '0'), (int) (buf[1] - '0'), 'O');
    }
    return true;
}

//Metoda zabezpecuje prenos tahu tam a prijatie tahu spat

bool CNetwork::MakeMove(void) {
    //pripravi buffer
    char buf[BUFFSIZE];
    int mlen = 2;
    int x, y;

    //Odosle tah
    std::cout << "Odosielam..." << std::endl;
    //ziska poslednu zmenu
    m_Table->GetLastChanged(x, y);
    buf[0] = (char) x + '0';
    buf[1] = (char) y + '0';
    //a odosle
    if (send(c_sockfd, buf, mlen, 0) == -1) {
        close(c_sockfd);
        return false;
    }

    //Kontrola ze ci nevyhral
    if (m_Table->Check() != '\0') return true;

    //prijme tah
    std::cout << "Prijmam..." << std::endl;
    buf[0] = '\0';
    mlen = 0;
    while (buf[0] == '\0') {
        if ((mlen = recv(c_sockfd, buf, BUFFSIZE, 0)) == -1) {
            close(c_sockfd);
            return false;
        }
    }

    if (buf[0] == 'E' && buf[1] == 'X') {
        return false;
    }
    //Spracuje tah do tabulky
    m_Table->SetChar((int) (buf[0] - '0'), (int) (buf[1] - '0'), 'O');
    return true;
}

//Odosle ukoncenie spojenia

void CNetwork::CloseConnection(void) {
    char buf[BUFFSIZE];
    int mlen = 2;

    buf[0] = 'E';
    buf[1] = 'X';

    send(c_sockfd, buf, mlen, 0);
}