/*
 * File:   CTable.cpp
 * Author: Adam Netocny
 *
 * Created on Pondelok, 2009, december 28, 17:49
 */

#include "CTable.h"

//Konstruktor, inicializuje pociatocne hodnoty a alokuje pole

CTable::CTable(int gridSize) {
    //inicializacia hodnot
    m_LastChanged[0] = -1;
    m_LastChanged[1] = -1;
    m_Size = gridSize;
    m_RealSize = gridSize;
    //Alokacia pola
    m_Array = new char*[m_RealSize];
    for (int i = 0; i < m_RealSize; i++) {
        m_Array[i] = new char[m_RealSize];
    }
    //inicializacia hodnot v poli
    for (int i = 0; i < m_RealSize; i++) {
        for (int j = 0; j < m_RealSize; j++) {
            m_Array[i][j] = '\0';
        }
    }
}

//Destruktor, uvolnuje pole

CTable::~CTable(void) {
    for (int i = 0; i < m_RealSize; i++) {
        delete[] m_Array[i];
    }
    delete[] m_Array;
}

//Metoda nastavi hodnotu na danej pozicii

bool CTable::SetChar(int x, int y, char val) {
    //ak je v danych suradniciach
    if (x >= m_Size || y >= m_Size || x < 0 || y < 0) return false;
    //ak nie je obsadene
    if (m_Array[x][y] != '\0') return false;
    //nastavi hodnotu a posledne zmenene suradnice
    m_Array[x][y] = val;
    m_LastChanged[0] = x;
    m_LastChanged[1] = y;
    return true;
}

//Metoda ziska hodnotu na danej pozicii

bool CTable::GetChar(int x, int y, char & val) {
    //ak je v danych suradniciach
    if (x >= m_Size || y >= m_Size || x < 0 || y < 0) return false;
    val = m_Array[x][y];
    return true;
}

//Metoda znuluje hodnotu na danych suradniciach

void CTable::ClearChar(int x, int y) {
    //ak je v danych suradniciach
    if (x >= m_Size || y >= m_Size || x < 0 || y < 0) return;
    m_Array[x][y] = '\0';
}

//Metoda nastavi velkost hracieho pola

void CTable::SetSize(int gridSize) {
    //Ak je pozadovana velkost vacsia
    if (m_RealSize < gridSize) {
        m_RealSize = gridSize;

        //Alokacia pola
        char** m_Array1 = new char*[m_RealSize];
        for (int i = 0; i < m_RealSize; i++) {
            m_Array1[i] = new char[m_RealSize];
        }

        //inicializacia hodnot v poli
        for (int i = 0; i < m_Size; i++) {
            for (int j = 0; j < m_Size; j++) {
                m_Array1[i][j] = m_Array[i][j];
            }
        }

        //uvolnime povodne pole
        for (int i = 0; i < m_RealSize; i++) {
            delete[] m_Array[i];
        }
        delete[] m_Array;

        //zamenime polia
        m_Array = m_Array1;
    }
    //nastavi logicku velkost
    m_Size = gridSize;
}

//Metoda ziska posledne zmenene koordinaty

void CTable::GetLastChanged(int & x, int & y) {
    x = m_LastChanged[0];
    y = m_LastChanged[1];
}

//Metoda kontroluje ci je nenastala vyhra, alebo remiza

char CTable::Check(void) {
    //pocet vytaznych
    int dlzka = ((m_Size < 5) ? m_Size - 1 : 4);

    //kontrola vo vodorovnom smere
    for (int i = 0; i < m_Size; i++) {
        for (int j = dlzka; j < m_Size; j++) {
            bool test = true;
            for (int x = dlzka; x > 0; x--) {
                test = test && m_Array[i][j] == m_Array[i][j - x];
            }
            if (test) {
                if (m_Array[i][j] != '\0')
                    return m_Array[i][j];
            }
        }
    }

    //kontrola vo horizontalnom smere
    for (int i = 0; i < m_Size; i++) {
        for (int j = dlzka; j < m_Size; j++) {
            bool test = true;
            for (int x = dlzka; x > 0; x--) {
                test = test && m_Array[j][i] == m_Array[j - x][i];
            }
            if (test) {
                if (m_Array[j][i] != '\0')
                    return m_Array[j][i];
            }
        }
    }

    //kontrola v jednom sikmom smere
    for (int i = 0; i < m_Size; i++) {
        for (int j = 0; j < m_Size; j++) {
            if (j + dlzka < m_Size && i + dlzka < m_Size) {
                bool test = true;
                for (int x = dlzka; x > 0; x--) {
                    test = test && m_Array[i][j] == m_Array[i + x][j + x];
                }
                if (test) {
                    if (m_Array[i][j] != '\0')
                        return m_Array[i][j];
                }
            }
        }
    }

    //kontrola v druhom sikmom smere
    for (int i = 0; i < m_Size; i++) {
        for (int j = 0; j < m_Size; j++) {
            if (j - dlzka >= 0 && i + dlzka < m_Size) {
                bool test = true;
                for (int x = dlzka; x > 0; x--) {
                    test = test && m_Array[i][j] == m_Array[i + x][j - x];
                }
                if (test) {
                    if (m_Array[i][j] != '\0')
                        return m_Array[i][j];
                }
            }
        }
    }

    //kontrola remizy
    bool test = true;
    for (int i = 0; i < m_Size; i++) {
        for (int j = 0; j < m_Size; j++) {
            if (m_Array[i][j] == '\0') {
                test = false;
            }
        }
    }
    if (test) return 'R';
    return '\0';
}

//Metoda ziska velkost pola
int CTable::GetSize(void) {
    return m_Size;
}
